/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File        : MainTask_GUI.c
---------------------------END-OF-HEADER------------------------------
*/

#include "DIALOG.h"
#include "Resource.h"
#include "WeatherStation.h"

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       _cbBk
*/
static void _cbBk(WM_MESSAGE * pMsg) {
  switch (pMsg->MsgId) {
  case WM_PAINT:
    GUI_SetBkColor(GUI_BLACK);
    GUI_Clear();
    break;
  default:
    WM_DefaultProc(pMsg);
    break;
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       MainTask_GUI()
*
* Function description
*   Task executed by the RTOS to create further resources and
*   running the main application.
*/
void MainTask_GUI(void) {
  WM_HWIN        hDlg3;
  int            XSize;
  int            YSize;
  UTIL_DATE_TIME Time;

  //
  // Initialize GUI
  //
  WM_SetCreateFlags(WM_CF_MEMDEV);
  GUI_Init();
  WM_MULTIBUF_Enable(1);
  WM_MOTION_Enable(1);
  GUI_SetOrientation(GUI_SWAP_XY | GUI_MIRROR_Y);    // Set vertical screen orientation
  XSize = LCD_GetXSize();
  YSize = LCD_GetYSize();
  WM_SetSize(WM_HBKWIN, XSize, YSize);               // Set screen size
  WM_SetCallback(WM_HBKWIN, _cbBk);
  CreateWindow1();                                   // Create window for displaying the current time
  CreateWindow2();                                   // Create window for displaying the current weather
  hDlg3 = CreateWindow3();                           // Create window for displaying the weather forecast
  WM_MOTION_SetMoveable(hDlg3, WM_CF_MOTION_X, 1);
  CreateWindow4();                                   // Create window for displaying a background color
  //
  // Application loop
  //
  while (1) {
    if(MAIN_TimeNeedsUpdate) {
      MAIN_TimeIsInitialized = UTIL_IPGetNTPTime(IP_INFO_GetNumInterfaces()-1, TIMEZONE_OFFSET);
      MAIN_TimeNeedsUpdate = 0;
    }
    Time = UTIL_GetTimeDateFromTimestamp(GLOBAL_Time.Seconds, 1900); // Get the current time
    GUI_UpdateTime(&Time);                                           // Update display with current time
    if (MAIN_TempNeedsUpdate) {
      GUI_UpdateWeatherNow();
      GUI_UpdateWeatherForecast();
      MAIN_TempNeedsUpdate = 0;
    }    
    GUI_Delay(100);
  }
}

/*************************** End of file ****************************/
