/*********************************************************************
*                      SEGGER Microcontroller GmbH                   *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*      (c) 2018  SEGGER Microcontroller GmbH                         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : WeatherStation.h
Purpose : Application specific header file
*/

#ifndef WEATHERSTATION_H            // Avoid multiple inclusion.
#define WEATHERSTATION_H

#include "IP.h"
#include "GUI.h"
#include "DIALOG.h"
#include "Resource.h"
#include "RTOS.h"
#include "BSP.h"


#if defined(__cplusplus)
  extern "C" {                // Make sure we have C-declarations in C++ programs.
#endif
/*********************************************************************
*
*       Defines, global, user-specific
*
**********************************************************************
*/

//
// Weather server
//
#define WEATHER_API_SERVER             "api.openweathermap.org"
#define WEATHER_APP_ID                 "5ba1e692f46b7761db7dfad25c6770ea"          // openweathermap APP ID
//
#define WEATHER_APP_CITY               "Monheim am Rhein,de"
#define WEATHER_APP_CITY_DISPLAY       "Monheim"
#define WEATHER_APP_WEATHER_NOW        "/data/2.5/weather?q="  WEATHER_APP_CITY ",de&units=metric&appid=" WEATHER_APP_ID
#define WEATHER_APP_WEATHER_FORECAST   "/data/2.5/forecast?q=" WEATHER_APP_CITY ",de&units=metric&appid=" WEATHER_APP_ID
//
#define  NUM_FORECASTS_PER_DAY         8
#define  NUM_DAYS_FORECAST             5

//
// Time server
//
#define NTP_SERVER                     "de.pool.ntp.org"
#define TIMEZONE_OFFSET_HOURS          (1)                                // German winter time (1). German summer time would be (2).
#define TIMEZONE_OFFSET                (TIMEZONE_OFFSET_HOURS * 60 * 60)  // Timezone offset to GMT
#define TIMER_UPDATE_PERIOD            100
#define TIME_UPDATE_PERIOD             (1000/TIMER_UPDATE_PERIOD) * 60 * 60

/*********************************************************************
*
*       Defines, global
*
**********************************************************************
*/
#define WEATHER_UPDATE_PERIOD           (1000 * 60 * 5) // Update period of weather data. Default: Check every 5 minutes
#define WEATHER_FAIIL_PERIOD            (1000 * 10)     // Update period when we encountered an error. Default: Check every 10 seconds

#define IP_STATUS_LED                   (3)

#define WEATHER_ICON_CLEAR_SKY          SUNNY
#define WEATHER_ICON_FEW_CLOUDS         PARTIALY_CLOUDY_DAY
#define WEATHER_ICON_SCATTERED_CLOUDS   CLOUDS
#define WEATHER_ICON_BROKEN_CLOUDS      PARTIALY_CLOUDY_DAY
#define WEATHER_ICON_SHOWER_RAIN        RAIN
#define WEATHER_ICON_RAIN               RAIN
#define WEATHER_ICON_THUNDERSTORM       CLOUD_LIGHTNING
#define WEATHER_ICON_SNOW               SNOW
#define WEATHER_ICON_MIST               FOG_DAY
#define WEATHER_ICON_UNKNOWN            FOG_DAY

/*********************************************************************
*
*       Types, global
*
**********************************************************************
*/

typedef struct {
  U16 Year;
  U16 Month;
  U16 Day;
  U16 Hour;
  U16 Minute;
  U16 Second;
} UTIL_DATE_TIME;

typedef struct {
  char acTemp[16];
  char acTempMin[16];
  char acTempMax[16];
  char acWeather[64];
  char acHumidity[8];
  char acDate[64];
  char acWeekday[16];
  char acWeatherIcon[8];
} WEATHER_STATUS;

typedef enum {
  SUNNY,
  CHANCE_OF_STORM,
  CLOUD_LIGHTNING,
  CLOUDS,
  FOG_DAY,
  PARTIALY_CLOUDY_DAY,
  RAIN,
  SNOW
} WEATHER_CONDITION;

/*********************************************************************
*
*       Global data
*
**********************************************************************
*/

#ifdef   EXTERN
  #undef EXTERN
#endif

#ifndef MAIN_C       // Defined in Main.c which includes this header beside other C-files
  #define EXTERN extern
#else
  #define EXTERN
#endif

EXTERN int              MAIN_TempNeedsUpdate;
EXTERN WEATHER_STATUS   MAIN_WeatherNow;
EXTERN WEATHER_STATUS   MAIN_aWeatherDay[5];
                        
EXTERN int              MAIN_IPIsConnected;

EXTERN IP_NTP_TIMESTAMP GLOBAL_Time;
EXTERN int              MAIN_TimeNeedsUpdate;
EXTERN int              MAIN_TimeIsInitialized;

#undef EXTERN

/*********************************************************************
*
*       API functions / Function prototypes
*
**********************************************************************
*/

void           GUI_UpdateWeatherNow          (void);
void           GUI_UpdateWeatherForecast     (void);
void           GUI_UpdateTime                (UTIL_DATE_TIME* pTime);

WM_HWIN        CreateWindow1                 (void);
WM_HWIN        CreateWindow2                 (void);
WM_HWIN        CreateWindow3                 (void);
WM_HWIN        CreateWindow4                 (void);
                                             
const char*    GetWeekdayFromUTC             (U32 Time);
U32            GetWeatherIconId              (const char* sIcon);
int            StringCompareEnd              (const char* s, const char* sMatch);
                                             
U32            UTIL_GetTimeDate              (void);
UTIL_DATE_TIME UTIL_GetTimeDateFromTimestamp (U32 Timestamp, U32 Epoche);
int            UTIL_PrintDateTime            (char* pBuffer, int NumBytesBuffer, U32 Timestamp, U32 Epoche);
int            UTIL_IPGetNTPTime             (int IFaceId, int timezone_offset);

#if defined(__cplusplus)
}                             // Make sure we have C-declarations in C++ programs.
#endif

#endif                        // Avoid multiple inclusion.

/*************************** End of file ****************************/
